Comm library                         {#mainpage}
============

## Description

It is a set of drivers for standard communication devices. This library offers a
generic communication device which can be used to create drivers for new
communication devices easily: it is only necessary to implement the low level,
operating system dependent functions, all the other features are already
implemented.

<details><summary>Detailed description</summary>
<p>


The main features of all communication devices are:

* Data is automatically received and stored into a data FIFO by an internal thread.
* An event is generated whenever there is new data into this FIFO. Read operations are non-blocking.
* An event is generated when an unexpected error happen.
* Only a few functions have to be implemented to generate a new driver.

The communication devices currently supported by this library include:

* **serial port**: This drivers provides a simple and easy to use yet
complete interface to a standard computer serial port. This driver allows
the user to open any available serial port, configure it as needed and send
and receive data to and from it. Some USB devices can also use this driver
(those that appear as a virtual COM port).

* **ftdi driver**: This driver is intended to be used to interface with
devices with any of the following FTDI chips:
            - FT2232H
            - FT4232H
            - FT232R
            - FT245R
            - FT2232
            - FT232B
            - FT245B
            - FT8U232AM
            - FT8U245AM
. This driver can also be used with USB devices that appear as virtual serial
ports in the host computer. In this case it is also possible to use the serial
port driver

* **sockets**: This driver provide an implementation of server and client
TCP sockets in Linux. UDP connections are not yet supported. The server
side can handle as many connections as needed, providing notifications for
the following event for each of the connections:
            - new connection
            - disconnection
            - reception of new data
. Both the client and server can be used with other implementations of the POSIX sockets.

* **can bus**: This driver provides a simple and easy to use interface for CAN bus communication protocol.
This driver allows the user to open and configure a CAN port and send and receive data to and from it.
It also provides the possibility of create a Virtual Can that dumps the content of a file on a specific rate.

</p>
</details>


## Installation

* Add the labrobotica repository if it is not already added:

Run the commands on _add repository_ and _add key_ from [labrobotica_how_to installation](https://gitlab.iri.upc.edu/labrobotica/labrobotica_how_to/-/blob/master/README.md#installation)

* Install the package:

``` sudo apt update && sudo apt install iri-comm-dev ```

## Scripts

The following scripts located in `./scripts/*.sh` are provided:

* **can.sh**: necessary when using a CAN to usb adapter
  - Dependency: [can-utils](https://elinux.org/Can-utils)

        sudo apt install can-utils
  - Usage:

        ./can.sh -d <device> -n <name>
          -d  specify device route. For example, /dev/ttyUSB0"
          -n  specify human friendly device's name. For example, can0"

* **add_manufacturer_to_group_udev.sh**: adds an udev rule to add manufacturer devices to specified system group.
  - Usage:

        ./add_manufacturer_to_group_udev.sh -m <manufacturer> -g <group> [-f <file>]
          -m  specify manufacturer. For example, Segway"
          -g  specify group. For example, dialout"
          -f  specify udev file. Optional. Default: 99-manufacturer-<manufacturer>-to-group-<group>.rules

* **unbind_ftdi_udev.sh**: adds an udev rule so specified VENDOR:PRODUCT:SERIAL devices do not load *ftdi_sio* driver.
  - Usage:

        ./unbind_ftdi_udev.sh -v <vendor> -p <product> -s <serial> [-f <file>]
          -v  specify vendor id.
          -p  specify product id.
          -s  specify serial number
          -f  specify udev file. Optional. Default 99-ftdi-unbind-by-serial.rules"

* **undo_unbind_ftdi_udev.sh**: undo the previous script, removing lines from udev rules regarding the specified SERIAL.
  - Usage:

        ./undo_unbind_ftdi_udev.sh -s <serial> [-f <file>]
          -s  specify serial number
          -f  specify udev file. Optional. Default 99-ftdi-unbind-by-serial.rules"


## Disclaimer  

Copyright (C) Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
Mantainer IRI labrobotics (labrobotica@iri.upc.edu)

This package is distributed in the hope that it will be useful, but without any warranty. It is provided "as is" without warranty of any kind, either expressed or implied, including, but not limited to, the implied warranties of merchantability and fitness for a particular purpose. The entire risk as to the quality and performance of the program is with you. should the program prove defective, the GMR group does not assume the cost of any necessary servicing, repair  or correction.

In no event unless required by applicable law the author will be liable to you for damages, including any general, special, incidental or consequential damages arising out of the use or inability to use the program (including but not limited to loss of data or data being rendered inaccurate or losses sustained by you or third parties or a failure of the program to operate with any other programs), even if the author has been advised of the possibility of such damages.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>

## For developers

<details><summary>click here</summary>
<p>

## Dependencies

This package requires of the following system libraries and packages

* [cmake](https://www.cmake.org "CMake's Homepage"), a cross-platform build system.
* [doxygen](http://www.doxygen.org "Doxygen's Homepage") and [graphviz](http://www.graphviz.org "Graphviz's Homepage") to generate the documentation.
* stdc++ and pthread libraries.

Under linux all of these utilities are available in ready-to-use packages.

Under MacOS most of the packages are available via [fink](http://www.finkproject.org/ "Fink's Homepage")

This package also requires of the following IRI libraries:

* [iriutils](https://gitlab.iri.upc.edu/labrobotica/algorithms/iriutils "iriutils gitlab page"), a set of basic tools.

## Compilation and installation from source

Clone this repository and create a build folder inside:

``` mkdir build ```

Inside the build folder execute the following commands:

``` cmake .. ```

The default build mode is DEBUG. That is, objects and executables include debug information.

The RELEASE build mode optimizes for speed. To build in this mode execute instead
``` cmake .. -DCMAKE_BUILD_TYPE=RELEASE ```

The release mode will be kept until next time cmake is executed.

``` make -j $(nproc)``` 

In case no errors are reported, the generated libraries (if any) will be located at the
_lib_ folder and the executables (if any) will be located at the _bin_ folder.

In order to be able to use the library, it it necessary to copy it into the system.
To do that, execute

``` make install ```

as root and the shared libraries will be copied to */usr/local/lib/iri/comm* directory
and the header files will be copied to */usr/local/include/iri/comm* dierctory. At
this point, the library may be used by any user.

To remove the library from the system, exceute

``` make uninstall ```

as root, and all the associated files will be removed from the system.

To generate the documentation execute the following command:

``` make doc ```

## How to use it

To use this library in another library or application, in the CMakeLists.txt file, first it is necessary to locate if the library has been installed or not using the following command

``` FIND_PACKAGE(comm) ```

In the case that the package is present, it is necessary to add the header files directory to the include directory path by using

``` INCLUDE_DIRECTORIES(${comm_INCLUDE_DIR}) ```

and it is also necessary to link with the desired libraries by using the following command

``` TARGET_LINK_LIBRARIES(<executable name> ${comm_LIBRARY}) ```

## Examples

There are several examples that show how to use the different classes.

</p>
</details>
