#!/bin/sh

echo "--- add_manufacturer_to_group_udev.sh ---"

MANUFACTURER=
GROUP=
FILE=
UDEV_PATH="/etc/udev/rules.d"

usage() {
  echo "Usage: $0 -m <manufacturer> -g <group> [-f <file>]"
  echo " -m  specify manufacturer id.     For example, Segway"
  echo " -g  specify group id.            For example, dialout"
  echo " -f  specify udev file. Optional. Default 99-manufacturer-<manufacturer>-to-group-<group>.rules"
  exit 1
  }
  
while getopts ":hm:g:f:" arg; do
  case $arg in
    m)  MANUFACTURER=$OPTARG
        ;;
    g)  GROUP=$OPTARG
        ;;
    f)  MANUAL_FILE=$OPTARG;;
    h) usage;;
    *) usage;;
  esac
done
shift $((OPTIND-1))

if [ -z "$MANUAL_FILE" ]
then
  FILE="99-manufacturer-${MANUFACTURER}-to-group-${GROUP}.rules"
else
  FILE=$MANUAL_FILE
fi

if [ -z "$MANUFACTURER" ]
then
  echo "Error: no manufacturer specified"
  usage
fi

if [ -z "$GROUP" ]
then
  echo "Error: no group specified"
  usage
fi

if [ -z "$FILE" ]
then
  echo "Error: empty file specified"
  usage
fi

FILE="$UDEV_PATH/$FILE"

echo "  Add udev rule so MANUFACTURER=${MANUFACTURER} devices are added to GROUP=${GROUP}."
echo "  Adding new lines to udev rule file: ${FILE}"

LINE0="SUBSYSTEM==\"usb\", ATTR{manufacturer}==\"${MANUFACTURER}\", GROUP=\"${GROUP}\""

sudo touch $FILE
grep -qF -- "$LINE0" "$FILE" || echo "$LINE0" | sudo tee -a "$FILE" > /dev/null

sudo service udev restart
echo "  Need to unplug-plug devices for changes to take effect"