#!/bin/sh

echo "--- unbind_ftdi_udev.sh ---"

VENDOR=
PRODUCT=
SERIAL=
FILE="99-ftdi-unbind-by-serial.rules"
UDEV_PATH="/etc/udev/rules.d"

usage() {
  echo "Usage: $0 -v <vendor> -p <product> -s <serial> [-f <file>]"
  echo " -v  specify vendor  id."
  echo " -p  specify product id."
  echo " -s  specify serial number."
  echo " -f  specify udev file. Optional. Default 99-ftdi-unbind-by-serial.rules"
  exit 1
  }
  
while getopts ":hv:p:s:f:" arg; do
  case $arg in
    v)  VENDOR=$OPTARG;;
    p) PRODUCT=$OPTARG;;
    s)  SERIAL=$OPTARG;;
    f)    FILE=$OPTARG;;
    h) usage;;
    *) usage;;
  esac
done
shift $((OPTIND-1))

if [ -z "$VENDOR" ]
then
  echo "Error: no vendor specified"
  usage
fi

if [ -z "$PRODUCT" ]
then
  echo "Error: no product specified"
  usage
fi

if [ -z "$SERIAL" ]
then
  echo "Error: no serial specified"
  usage
fi

if [ -z "$FILE" ]
then
  echo "Error: empty file specified"
  usage
fi

FILE="$UDEV_PATH/$FILE"

echo "  Add udev rule so VENDOR:PRODUCT:SERIAL=${VENDOR}:${PRODUCT}:${SERIAL} devices do not load ftdi_sio driver."
echo "  Adding new lines to udev rule file: ${FILE}"

LINE0="#The following 2 lines make VENDOR:PRODUCT:SERIAL=${VENDOR}:${PRODUCT}:${SERIAL} devices not load ftdi_sio driver"
LINE1="ATTRS{idVendor}==\"${VENDOR}\", ATTRS{idProduct}==\"${PRODUCT}\", ATTRS{serial}==\"${SERIAL}\", MODE:=\"0666\""
LINE2="ATTRS{idVendor}==\"${VENDOR}\", ATTRS{idProduct}==\"${PRODUCT}\", ATTRS{serial}==\"${SERIAL}\", RUN+=\"/bin/sh -c 'echo \$kernel > /sys/bus/usb/drivers/ftdi_sio/unbind'\""

sudo touch $FILE
grep -qF -- "$LINE0" "$FILE" || echo "$LINE0" | sudo tee -a "$FILE" > /dev/null
grep -qF -- "$LINE1" "$FILE" || echo "$LINE1" | sudo tee -a "$FILE" > /dev/null
grep -qF -- "$LINE2" "$FILE" || echo "$LINE2" | sudo tee -a "$FILE" > /dev/null

sudo service udev restart
echo "  Need to unplug-plug devices for changes to take effect"